/*

Conversation widget for kdm greeter

Copyright (C) 1997, 1998, 2000 Steffen Hansen <hansen@kde.org>
Copyright (C) 2000-2003 Oswald Buddenhagen <ossi@kde.org>
Copyright (C) 2009 Jaroslav Barton <djaara@djaara.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "kgreet_fprintd.h"

#include <kglobal.h>
#include <klocale.h>
#include <klineedit.h>
#include <kuser.h>

#include <QLabel>
#include <QTimer>
#include <QGridLayout>


KFprintGreeter::KFprintGreeter(KGreeterPluginHandler *handler,
                               QWidget *parent,
                               const QString &fixedEntity,
                               Function func, Context ctx) 
    :QObject(),
	 KGreeterPlugin(handler),
     m_animLabel(0),
     m_loginLabel(0),
     m_fprintStatus(0),
     m_loginEdit(0),
     m_timer(0),
     m_sensor("swipe"),
	 m_fixedUser(fixedEntity),
	 m_func(func),
	 m_ctx(ctx),
	 authStarted( false )
{
	QGridLayout *gridLayout = 0;

	if (!handler->gplugHasNode("user-entry") 
        || !handler->gplugHasNode("anim-label") 
        || !handler->gplugHasNode("fprint-status")) {
		parent = new QWidget(parent);
		parent->setObjectName("talker");
		widgetList << parent;
		gridLayout = new QGridLayout( parent );
		gridLayout->setMargin( 0 );
	}
	
	if (m_ctx == ExUnlock) {
		m_fixedUser = KUser().loginName();
    }

	if (m_func != ChAuthTok) {
        int line = 0;
		
		m_timer = new QTimer();
		m_timer->setSingleShot(true);
		QObject::connect(m_timer, SIGNAL(timeout()), this, SLOT(timeoutSlot()));
        
		if (m_fixedUser.isEmpty()) {
			m_loginEdit = new KLineEdit( parent );
			m_loginEdit->setContextMenuPolicy( Qt::NoContextMenu );
			connect( m_loginEdit, SIGNAL(editingFinished()), SLOT(slotChanged()) );
			connect( m_loginEdit, SIGNAL(textChanged( const QString & )), SLOT(slotChanged()) );
			connect( m_loginEdit, SIGNAL(selectionChanged()), SLOT(slotChanged()) );
			if (!gridLayout) {
				m_loginEdit->setObjectName( "user-entry" );
				widgetList << m_loginEdit;
			} else {
				m_loginLabel = new QLabel( i18n( "&Username:" ), parent );
				m_loginLabel->setBuddy( m_loginEdit );
				gridLayout->addWidget( m_loginLabel, line, 0 );
				gridLayout->addWidget( m_loginEdit, line++, 1 );
			}
		} else if (m_ctx != Login && m_ctx != Shutdown && m_ctx != ExUnlock && gridLayout)        {
			m_loginLabel = new QLabel( i18n( "Username:" ), parent );
			gridLayout->addWidget( m_loginLabel, line, 0 );
			gridLayout->addWidget( new QLabel( m_fixedUser, parent ), line++, 1 );
		}
		
		m_fprintStatus = new QLabel();
		if (!gridLayout) {
			m_fprintStatus->setObjectName( "fprint-status" );
			widgetList << m_fprintStatus;
		} else{
			gridLayout->addWidget( m_fprintStatus, line++, 0, 1, 2 );
		}

		m_animLabel = new SensorAnimLabel();
		if (!gridLayout) {
			m_animLabel->setObjectName( "anim-label" );
			widgetList << m_animLabel;
		} else {
			gridLayout->addWidget( m_animLabel, 0, 2, 2, 1 );
		}
		
		if (m_loginEdit) {
			m_loginEdit->setFocus();
		}
		
		if (m_ctx == ExUnlock) {
			m_fprintStatus->setText( i18n( "Click on Unlock button." ) );
		}
	}
}

// virtual
KFprintGreeter::~KFprintGreeter()
{
	abort();
	qDeleteAll( widgetList );
}

void // virtual
KFprintGreeter::loadUsers( const QStringList &users )
{
	KCompletion *userNamesCompletion = new KCompletion;
	userNamesCompletion->setItems( users );
	m_loginEdit->setCompletionObject( userNamesCompletion );
	m_loginEdit->setAutoDeleteCompletionObject( true );
	m_loginEdit->setCompletionMode( KGlobalSettings::CompletionAuto );
}

void // virtual
KFprintGreeter::presetEntity( const QString &entity, int field )
{
	m_loginEdit->setText( entity );

	if (field == 0) {
		m_loginEdit->setFocus();
		m_loginEdit->selectAll();
	}

	m_curUser = entity;
}

QString // virtual
KFprintGreeter::getEntity() const
{
	return m_fixedUser.isEmpty() ? m_loginEdit->text() : m_fixedUser;
}

void // virtual
KFprintGreeter::setUser( const QString &user )
{
	m_curUser = user;
	m_loginEdit->setText( user );
}

void // virtual
KFprintGreeter::setEnabled( bool enable )
{
    m_loginLabel->setEnabled( enable );
	m_fprintStatus->setEnabled( enable );
	setActive( enable );
}

void // private
KFprintGreeter::returnData()
{
	handler->gplugReturnText((m_loginEdit ? m_loginEdit->text() :
			                  m_fixedUser).toLocal8Bit(), 
                              KGreeterPluginHandler::IsUser );
}

bool // virtual
KFprintGreeter::textMessage( const char *text, bool err )
{
	QString msg = QString(text);
    QString sensor;
    if (msg.contains(QString("swipe"), Qt::CaseInsensitive)) {
        sensor = "swipe";
    } else if (msg.contains(QString("place"), Qt::CaseInsensitive)) {
        sensor = "place";
    }
    
    if (m_sensor != sensor) {
        m_sensor = sensor;
        m_animLabel->loadMovie(m_sensor);
    }

	if (err) {
        // stop authentication
        abort();
    } else { 
        showText(msg);
    }
        
    return true;
}

void // virtual
KFprintGreeter::textPrompt( const char *prompt, bool echo, bool nonBlocking )
{
	Q_UNUSED(echo);
	Q_UNUSED(nonBlocking);
	QString text = QString(prompt);

    if (text.contains(QString("Password"), Qt::CaseInsensitive)) {
        // stop authentication
        abort();
    }
}

bool // virtual
KFprintGreeter::binaryPrompt( const char *, bool )
{
	// this simply cannot happen ... :}
	return true;
}

void // virtual
KFprintGreeter::start()
{
    // start an authentication cycle
    authStarted = false;
}

void // virtual
KFprintGreeter::suspend()
{
}

void // virtual
KFprintGreeter::resume()
{
}

void // virtual
KFprintGreeter::next()
{
	authStarted = true;
	setActive(false);
	handler->gplugStart();
	returnData();
    m_timer->start(500);
}

void // virtual
KFprintGreeter::abort()
{
	if ( authStarted ) {
		handler->gplugReturnText( 0, 0 );
	}

	m_animLabel->stop();
    m_fprintStatus->setText(i18n("Authentication failed. Please try again."));
    setActive(true);
    authStarted = false;
}

void // virtual
KFprintGreeter::succeeded()
{
	if (!authTok) {
		setActive( false );
	}

	authStarted = false;
	m_animLabel->stop();
	m_fprintStatus->setText(i18n("Succeeded"));
}

void // virtual
KFprintGreeter::failed()
{
	setActive( false );
 	m_fprintStatus->clear();
	m_animLabel->stop();
	authStarted = false;
}

void // virtual
KFprintGreeter::revive()
{
	setActive( true );
	m_loginEdit->setFocus();
}

void // virtual
KFprintGreeter::clear()
{
	if (m_loginEdit) {
		m_loginEdit->clear();
		m_loginEdit->setFocus();
		m_curUser.clear();
	}

	m_fprintStatus->clear();
    m_animLabel->stop();
}

void KFprintGreeter::showText(const QString &text) {
    m_fprintStatus->setText(text);
}

void KFprintGreeter::timeoutSlot() {
	if ( authStarted && m_animLabel ) {
        m_animLabel->start();
	} else {
        m_animLabel->stop();
    } 
}

void
KFprintGreeter::setActive( bool enable )
{
	if (m_loginEdit) {
		m_loginEdit->setEnabled( enable );
    }
}

void
KFprintGreeter::slotChanged()
{
	if (authStarted) {
		m_loginEdit->setText( m_loginEdit->text().trimmed() );
		
		if (m_curUser != m_loginEdit->text()) {
			m_curUser = m_loginEdit->text();
			handler->gplugSetUser( m_curUser );
		    handler->gplugChanged();
		}
	}
}

static bool init( const QString &,
                  QVariant (*getConf)( void *, const char *, const QVariant & ),
                  void *ctx )
{
	Q_UNUSED(getConf);
	Q_UNUSED(ctx);
	KGlobal::locale()->insertCatalog( "kgreet_fprintd" );
	return true;
}

static void done( void )
{
	KGlobal::locale()->removeCatalog( "kgreet_fprintd" );
}

static KGreeterPlugin *
create( KGreeterPluginHandler *handler,
        QWidget *parent,
        const QString &fixedEntity,
        KGreeterPlugin::Function func,
        KGreeterPlugin::Context ctx )
{
	return new KFprintGreeter( handler, parent, fixedEntity, func, ctx );
}

KDE_EXPORT KGreeterPluginInfo kgreeterplugin_info = {
	I18N_NOOP2("@item:inmenu authentication method", "Username + Fingerprint"), "fprintd",
	KGreeterPluginInfo::Local | KGreeterPluginInfo::Presettable,
	init, done, create
};

#include "kgreet_fprintd.moc"
